@@ -150,6 +150,7 @@ setMethod("getDesign", "unmarkedMultFrame",
     M <- numSites(umf)
     R <- obsNum(umf)
     nY <- umf@numPrimary
+    J <- R / nY
   
     ## Compute phi design matrices
     if(is.null(umf@yearlySiteCovs)) {
@@ -157,17 +158,50 @@ setMethod("getDesign", "unmarkedMultFrame",
     } else {
         yearlySiteCovs <- umf@yearlySiteCovs
     }
+    ## add siteCovs in so they can be used as well
+    if(!is.null(umf@siteCovs)) {
+        sC <- umf@siteCovs[rep(1:M, each = nY),,drop=FALSE]
+        yearlySiteCovs <- cbind(yearlySiteCovs, sC)
+        }
+  
+    ## Compute site-level design matrix for psi
+    if(is.null(siteCovs(umf)))
+        siteCovs <- data.frame(placeHolder = rep(1, M))
+    else
+        siteCovs <- siteCovs(umf)
+
+    W.mf <- model.frame(psiformula, siteCovs, na.action = NULL)
+    if(!is.null(model.offset(W.mf)))
+        stop("offsets not currently allowed in colext", call.=FALSE)
+    W <- model.matrix(psiformula, W.mf)
+
+  
+    ## Compute detection design matrix
+    if(is.null(obsCovs(umf)))
+        obsCovs <- data.frame(placeHolder = rep(1, M*R))
+    else
+        obsCovs <- obsCovs(umf)
+
+    ## add site and yearlysite covariates, which contain siteCovs
+    cnames <- c(colnames(obsCovs), colnames(yearlySiteCovs))
+    obsCovs <- cbind(obsCovs, yearlySiteCovs[rep(1:(M*nY), each = J),])
+    colnames(obsCovs) <- cnames
+	
+    ## add observation number if not present
+    if(!("obsNum" %in% names(obsCovs)))
+        obsCovs <- cbind(obsCovs, obsNum = as.factor(rep(1:R, M)))
+	
+    V.mf <- model.frame(detformula, obsCovs, na.action = NULL)
+    if(!is.null(model.offset(V.mf)))
+        stop("offsets not currently allowed in colext", call.=FALSE)
+    V <- model.matrix(detformula, V.mf)
+    
     ## in order to drop factor levels that only appear in last year,
     ## replace last year with NAs and use drop=TRUE
     yearlySiteCovs[seq(nY,M*nY,by=nY),] <- NA
     yearlySiteCovs <- as.data.frame(lapply(yearlySiteCovs, function(x) {
         x[,drop = TRUE]
         }))
-    ## add siteCovs in so they can be used as well
-    if(!is.null(umf@siteCovs)) {
-        sC <- umf@siteCovs[rep(1:M, each = nY),,drop=FALSE]
-        yearlySiteCovs <- cbind(yearlySiteCovs, sC)
-        }
     X.mf.gam <- model.frame(gamformula, yearlySiteCovs, na.action = NULL)
     if(!is.null(model.offset(X.mf.gam)))
         stop("offsets not currently allowed in colext", call.=FALSE)
@@ -176,57 +210,16 @@ setMethod("getDesign", "unmarkedMultFrame",
     if(!is.null(model.offset(X.mf.eps)))
         stop("offsets not currently allowed in colext", call.=FALSE)
     X.eps <- model.matrix(epsformula, X.mf.eps)
-  
-    ## Compute site-level design matrix for psi
-    if(is.null(siteCovs(umf))) {
-        siteCovs <- data.frame(placeHolder = rep(1, M))
-    } else {
-        siteCovs <- siteCovs(umf)
-    }
-    W.mf <- model.frame(psiformula, siteCovs, na.action = NULL)
-    if(!is.null(model.offset(W.mf)))
-        stop("offsets not currently allowed in colext", call.=FALSE)
-    W <- model.matrix(psiformula, W.mf)
+    
+	
+    if(na.rm)
+        out <- handleNA(umf, X.gam, X.eps, W, V)
+    else
+        out <- list(y=getY(umf), X.gam=X.gam, X.eps=X.eps, W=W, V=V, 
+            removed.sites=integer(0))
 
-    #  ## impute missing yearlySiteCovs across years as average
-    #  X <- t(apply(X, 1, function(x) {
-    #            out <- x
-    #            out[is.na(x)] <- mean(x)
-    #          }))
-  
-	## Compute detection design matrix
-	if(is.null(obsCovs(umf))) {
-		obsCovs <- data.frame(placeHolder = rep(1, M*R))
-	} else {
-		obsCovs <- obsCovs(umf)
-	}
-	
-	## add site and yearlysite covariates at observation-level
-	## DOUBLE CHECK THIS. Shouldn't it be:
-  ## yearlySiteCovs[rep(1:(M*nY), each = J),] where J is R/nY
-	obsCovs <- cbind(obsCovs, yearlySiteCovs[rep(1:(M*nY), each = R),],
-                         siteCovs[rep(1:M, each = R), ])
-	
-	## add observation number if not present
-	if(!("obsNum" %in% names(obsCovs))) {
-		obsCovs <- cbind(obsCovs, obsNum = as.factor(rep(1:R, M)))
-	}
-	
-	V.mf <- model.frame(detformula, obsCovs, na.action = NULL)
-  if(!is.null(model.offset(V.mf)))
-        stop("offsets not currently allowed in colext", call.=FALSE)
-	V <- model.matrix(detformula, V.mf)
-	
-	if(na.rm)
-		out <- handleNA(umf, X.gam, X.eps, W, V)
-	else
-		out <- list(y=getY(umf), X.gam=X.gam, X.eps=X.eps,
-                            W=W,V=V,
-				removed.sites=integer(0))
-	
-	return(list(y = out$y, X.eps = out$X.eps, X.gam = out$X.gam,
-                    W = out$W, V = out$V,
-                    removed.sites = out$removed.sites))
+    return(list(y = out$y, X.eps = out$X.eps, X.gam = out$X.gam, W = out$W, 
+        V = out$V, removed.sites = out$removed.sites))
 })
 
 
@@ -275,7 +268,6 @@ setMethod("handleNA", "unmarkedMultFrame", function(umf, X.gam, X.eps, W, V)
 	
     y <- matrix(y.long, M, numY(umf), byrow = TRUE)
     sites.to.remove <- apply(y, 1, function(x) all(is.na(x)))
-	
     num.to.remove <- sum(sites.to.remove)
     if(num.to.remove > 0) {
         y <- y[!sites.to.remove, ,drop = FALSE]
@@ -288,11 +280,226 @@ setMethod("handleNA", "unmarkedMultFrame", function(umf, X.gam, X.eps, W, V)
     list(y = y, X.gam = X.gam, X.eps = X.eps, W = W, V = V,
         removed.sites = which(sites.to.remove))
 })
+
+  
+
+
+
+
+# pcountOpen
+setMethod("getDesign", "unmarkedFramePCO", 
+    function(umf, formula, na.rm = TRUE) 
+{
+    aschar1 <- as.character(formula)
+    aschar2 <- as.character(formula[[2]])
+    aschar3 <- as.character(formula[[2]][[2]])    
+
+    pformula <- as.formula(paste(aschar1[1], aschar1[3]))
+    omformula <- as.formula(paste(aschar2[1], aschar2[3]))
+    gamformula <- as.formula(paste(aschar3[1], aschar3[3]))
+    lamformula <- as.formula(formula[[2]][[2]][[2]])
+	
+    y <- getY(umf)
+    M <- nrow(y)
+    T <- umf@numPrimary
+    J <- ncol(y) / T
+    delta <- umf@primaryPeriod
+    
+    if(is.null(umf@yearlySiteCovs))
+        yearlySiteCovs <- data.frame(placeHolder = rep(1, M*T))
+    else
+        yearlySiteCovs <- umf@yearlySiteCovs
+    
+    ## add siteCovs in so they can be used as well
+    if(!is.null(umf@siteCovs)) {
+        sC <- umf@siteCovs[rep(1:M, each = T),,drop=FALSE]
+        yearlySiteCovs <- cbind(yearlySiteCovs, sC)
+        }
+    
+    if(is.null(siteCovs(umf)))
+	     siteCovs <- data.frame(placeHolder = rep(1, M))
+    else
+	     siteCovs <- siteCovs(umf)
+
+    Xlam.mf <- model.frame(lamformula, siteCovs, na.action = NULL)
+    Xlam <- model.matrix(lamformula, Xlam.mf)
+
+    if(is.null(obsCovs(umf)))
+	     obsCovs <- data.frame(placeHolder = rep(1, M*J*T))
+    else
+	     obsCovs <- obsCovs(umf)
+	
+    colNames <- c(colnames(obsCovs), colnames(yearlySiteCovs))
+	
+	  # Add yearlySiteCovs, which contains siteCovs
+    obsCovs <- cbind(obsCovs, yearlySiteCovs[rep(1:(M*T), each = J),])
+    colnames(obsCovs) <- colNames
+	
+    if(!("obsNum" %in% names(obsCovs)))
+        obsCovs <- cbind(obsCovs, obsNum = as.factor(rep(1:(J*T), M)))
+        
+    # Ignore last year of data
+    transCovs <- yearlySiteCovs[-seq(T, M*T, by=T),,drop=FALSE]
+    for(i in 1:ncol(transCovs))
+        if(is.factor(transCovs[,i])) 
+            transCovs[,i] <- factor(transCovs[,i]) # drop unused levels
+	
+    Xp.mf <- model.frame(pformula, obsCovs, na.action = NULL)
+    Xp <- model.matrix(pformula, Xp.mf)
+    Xgam.mf <- model.frame(gamformula, transCovs, na.action = NULL)
+    Xgam <- model.matrix(gamformula, Xgam.mf)
+    Xom.mf <- model.frame(omformula, transCovs, na.action = NULL)
+    Xom <- model.matrix(omformula, Xom.mf)
+    
+    # determine if gamma and omega are scalar, vector, or matrix valued
+    # Runtime is much faster for scalars and vectors
+    Xgo <- cbind(Xgam, Xom)
+    getGOdims <- function(x) {
+        xm <- matrix(x, M, T-1, byrow=TRUE)
+        if(all(dim(unique(xm, MARGIN=1)) == c(1, T-1)))
+            return("rowvec")
+        else if(all(dim(unique(xm, MARGIN=2)) == c(M, 1)))
+            return("colvec")
+        else return("matrix")
+        }
+    if(isTRUE(all.equal(gamformula, ~1)) & isTRUE(all.equal(omformula, ~1)))
+        go.dims <- "scalar"
+    else {
+        go.dims.vec <- apply(Xgo, 2, getGOdims)
+        if(all(go.dims.vec == "rowvec"))
+            go.dims <- "rowvec"
+        else if(all(go.dims.vec == "colvec"))
+            go.dims <- "colvec"
+        else
+            go.dims <- "matrix"
+        }
+	
+    if(na.rm)
+        out <- handleNA(umf, Xlam, Xgam, Xom, Xp, delta)
+    else {   # delta needs to be formatted first
+        ya <- array(y, c(M, J, T))
+        yna <- apply(is.na(ya), c(1,3), all)
+        delta <- formatDelta(delta, yna)
+        out <- list(y=y, Xlam=Xlam, Xgam=Xgam, Xom=Xom, Xp=Xp, 
+            delta=delta, removed.sites=integer(0), go.dims=go.dims)
+        }
+	
+    return(list(y = out$y, Xlam = out$Xlam, Xgam = out$Xgam, Xom = out$Xom, 
+        Xp = out$Xp, delta = out$delta, removed.sites = out$removed.sites, 
+        go.dims = go.dims))
+    })
+    
+  
     
     
     
     
+
+
+setMethod("handleNA", "unmarkedFramePCO", 
+    function(umf, Xlam, Xgam, Xom, Xp, delta) 
+{
+	obsToY <- obsToY(umf)
+	if(is.null(obsToY)) stop("obsToY cannot be NULL to clean data.")
+
+	M <- numSites(umf)	
+	T <- umf@numPrimary
+	y <- getY(umf)
+	J <- ncol(y) / T
+
+	Xlam.long <- Xlam[rep(1:M, each = J*T),]
+	Xlam.long.na <- is.na(Xlam.long)
+	
+	long.na <- function(x) {
+		x.mat <- matrix(x, M, J*T, byrow = TRUE)
+		x.mat <- is.na(x.mat)
+		x.mat <- x.mat %*% obsToY
+		x.long <- as.vector(t(x.mat))
+		x.long > 0
+		}
+
+  o2y2 <- diag(T)
+  o2y2 <- o2y2[-T, -T]
+
+	long.na2 <- function(x) {
+		x.mat <- matrix(x, M, T-1, byrow = TRUE)
+		x.mat <- is.na(x.mat)		
+		x.mat <- x.mat %*% o2y2
+		x.long <- as.vector(t(x.mat))
+		x.long > 0
+		}
+	
+	Xp.long.na <- apply(Xp, 2, long.na)
+	Xp.long.na <- apply(Xp.long.na, 1, any)
+	
+  Xgam.long.na <- apply(Xgam, 2, long.na2)			
+	Xgam.long.na <- apply(Xgam.long.na, 1, any)		
+	Xom.long.na <- apply(Xom, 2, long.na2)			
+	Xom.long.na <- apply(Xom.long.na, 1, any)		
+	
+	y.long <- as.vector(t(y))
+	y.long.na <- is.na(y.long)
+	
+#  delta.long <- as.vector(t(delta))
+#	delta.long.na <- is.na(delta.long)
+	
+	covs.na <- apply(cbind(Xlam.long.na, Xp.long.na), 1, any)
+	covs.na2 <- apply(cbind(Xgam.long.na, Xom.long.na), 1, any)
+	covs.na3 <- rep(covs.na2, each=J)
+	# If gamma[1, 1] is NA, remove y[1, 2]
+	#common <- 1:(M*J*(T-1))
+  ignore <- rep(seq(1, M*J*T, by=J*T), each=J) + 0:(J-1)
+  covs.na[-ignore] <- covs.na[-ignore] | covs.na3 
+	
+	## are any NA in covs not in y already?
+	y.new.na <- covs.na & !y.long.na      
+	
+	if(sum(y.new.na) > 0) {
+        y.long[y.new.na] <- NA
+        warning("Some observations have been discarded because corresponding covariates were missing.", call. = FALSE)
+        }
+	
+	y.wide <- matrix(y.long, nrow=M, ncol=J*T, byrow = TRUE)
+#	delta <- matrix(delta.long, nrow=M, ncol=T, byrow = TRUE)
+	sites.to.remove <- apply(y.wide, 1, function(x) all(is.na(x)))
+  # Should also remove sites with no omega and gamma before an observation
+  # remove all observations before the one after the first real omega/gamma
+#  covs.na2.mat <- matrix(covs.na2, M, T-1, byrow=TRUE)
+#  last.y <- apply(y, 1, function(x) max(which(!is.na(y))))
+#  last.go <- apply(covs.na2.mat, 1, function(x)
+#      if(any(x)) {
+#          if(all(x))
+#              return(0)
+#          else
+#              return(max(which(!x)))
+#          }
+#      else
+#          return(T-1))
+#  no.go.before.y <- last.y <= last.go
+          
+	ya <- array(y.wide, c(M, J, T))
+	yna <- apply(is.na(ya), c(1,3), all)
+	delta <- formatDelta(delta, yna)
+	
+	num.to.remove <- sum(sites.to.remove)
+	if(num.to.remove > 0) {
+		y.wide <- y.wide[!sites.to.remove, ,drop = FALSE]
+		Xlam <- Xlam[!sites.to.remove, ,drop = FALSE]
+		Xgam <- Xgam[!sites.to.remove[rep(1:M, each = T-1)], ,drop = FALSE]
+		Xom <- Xom[!sites.to.remove[rep(1:M, each = T-1)], ,drop = FALSE]
+		Xp <- Xp[!sites.to.remove[rep(1:M, each = T)], ,drop = FALSE]
+		delta <- delta[!sites.to.remove, ,drop =FALSE]
+		warning(paste(num.to.remove, "sites have been discarded because of missing data."), call.=FALSE)
+	}
+	
+	list(y = y.wide, Xlam = Xlam, Xgam = Xgam, Xom = Xom, Xp = Xp, 
+		delta = delta, removed.sites = which(sites.to.remove))
+})
+
+
     
+
+
     
     
     
@@ -301,8 +508,6 @@ setMethod("handleNA", "unmarkedMultFrame", function(umf, X.gam, X.eps, W, V)
 # UnmarkedFrameGMN    
 	
 
-
-
 setMethod("getDesign", "unmarkedFrameGMM", 
     function(umf, formula, na.rm = TRUE) 
 {
@@ -350,9 +555,10 @@ setMethod("getDesign", "unmarkedFrameGMM",
         obsCovs <- data.frame(placeHolder = rep(1, M*R))
     } else obsCovs <- obsCovs(umf)
 	
-    # add site and yearlysite covariates at observation-level
-    obsCovs <- cbind(obsCovs, yearlySiteCovs[rep(1:(M*T), each = J),],
-        siteCovs[rep(1:M, each = R), ])
+    # add site and yearlysite covariates, which contain siteCovs
+    cnames <- c(colnames(obsCovs), colnames(yearlySiteCovs))
+    obsCovs <- cbind(obsCovs, yearlySiteCovs[rep(1:(M*T), each = J),])
+    colnames(obsCovs) <- cnames
 	
     # add observation number if not present
     if(!("obsNum" %in% names(obsCovs)))
@@ -380,7 +586,6 @@ setMethod("getDesign", "unmarkedFrameGMM",
 
 
 
-
 setMethod("handleNA", "unmarkedFrameGMM", 
     function(umf, Xlam, Xlam.offset, Xphi, Xphi.offset, Xdet, Xdet.offset) 
 {
@@ -441,4 +646,4 @@ setMethod("handleNA", "unmarkedFrameGMM",
         Xphi.offset = Xphi.offset, Xdet = Xdet, Xdet.offset = Xdet.offset,
         removed.sites = which(sites.to.remove))
 })
-    
+  
