@@ -38,4 +38,30 @@ test.umfDS.args <- function() {
     checkException(umf <- unmarkedFrameDS(y=y, siteCovs=sc, survey=s, 
         dist.breaks=1:3, unitsIn=uin))
              
-    }    
\ No newline at end of file
+    }
+    
+    
+
+test.obsToY <- function() {
+    y <- matrix(c(
+        1, 0, 0,
+        2, 1, 0,
+        1, 0, 1,
+        2, 1, 2,
+        1, 0, 3,
+        1, 1, 1), nrow=6, ncol=3, byrow=TRUE)
+    oc <- matrix(c(
+        1, 0,
+        2, 1,
+        1, 1,
+        NA, 0,
+        1, NA,
+        NA, NA), nrow=6, ncol=2, byrow=TRUE)
+    umf <- unmarkedFrameMPois(y = y, obsCovs = list(x=oc), type="double")
+    o2y <- obsToY(umf)
+    
+    checkEquals(o2y, matrix(1, 2, 3))
+    oc.na <- is.na(oc)
+    oc.na %*% o2y
+    
+    }    
